local L = AceLibrary("AceLocale-2.2"):new("KingOfTheJungle")
local PT = AceLibrary:GetInstance("PeriodicTable-2.0")
local BS = AceLibrary:GetInstance("Babble-Spell-2.2")
local PL = ParserLib:GetInstance("1.1")

KingOfTheJungle = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0")

KingOfTheJungle:RegisterDB("KingOfTheJungleDB", "KingOfTheJungleDBPerChar")

KingOfTheJungle:RegisterDefaults('char', {
	warnPetHungry = true,
	petHappinessLevel = 2,
    msgInterval = 60,
	lastWarningTime = 0,
	buttonX = 100,
	buttonY = 0,
	buttonScale = 1.2,
	buttonPoint = "CENTER",
})

KingOfTheJungle:RegisterDefaults('profile', {
	foodlist = nil
})

function KingOfTheJungle:InitDB()	
	local convert = function(t_old)
		if t_old then
			local t = {}			
			for k, v in pairs(t_old) do
				local f = {}
				f["id"] = k
				f["value"] = v
				f["selected"] = true
				
				table.insert(t,f)
			end
			
			table.sort(t, function (a,b) return a.value > b.value end)
			
			return t
		end
		
		return nil
	end
	
	if self.db.profile.foodlist == nil then
		self.db.profile.foodlist = {}
		
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Meat Raw", ["list"] = convert(PT:GetMultiSetTable("Food - Meat Raw"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Meat", ["list"] = convert(PT:GetMultiSetTable("Food - Meat"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Meat Bonus", ["list"] = convert(PT:GetMultiSetTable("Food - Meat Bonus"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Meat Stats", ["list"] = convert(PT:GetMultiSetTable("Food - Meat Stats"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Bread", ["list"] = convert(PT:GetMultiSetTable("Food - Bread"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Bread Bonus", ["list"] = convert(PT:GetMultiSetTable("Food - Bread Bonus"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Bread Conjured", ["list"] = convert(PT:GetMultiSetTable("Food - Bread Conjured"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Cheese", ["list"] = convert(PT:GetMultiSetTable("Food - Cheese"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Cheese Bonus", ["list"] = convert(PT:GetMultiSetTable("Food - Cheese Bonus"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Fish Raw", ["list"] = convert(PT:GetMultiSetTable("Food - Fish Raw"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Fish", ["list"] = convert(PT:GetMultiSetTable("Food - Fish"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Fish Bonus", ["list"] = convert(PT:GetMultiSetTable("Food - Fish Bonus"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Fish Stats", ["list"] = convert(PT:GetMultiSetTable("Food - Fish Stats"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Fruit", ["list"] = convert(PT:GetMultiSetTable("Food - Fruit"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Fruit Stats", ["list"] = convert(PT:GetMultiSetTable("Food - Fruit Stats"))})
		table.insert(self.db.profile.foodlist,{["diet"] = "Food - Fungus", ["list"] = convert(PT:GetMultiSetTable("Food - Fungus"))})
	end
end

function KingOfTheJungle:OnInitialize()	
	local options = { 
		type = "group",
			args = {				
				menu = {
					name = L["Show Config"], desc = L["Shows Config Screen"], type = 'execute',
					func = function() self:KOTJUI_Show() end
				},
				resetdb = {
					name = L["Reset Food"], desc = L["Reset Food Database"], type = 'execute',
					func = function() 
						self.db.profile.foodlist = nil  
						self:InitDB()
					end
				}
			}
		}

	KingOfTheJungle:RegisterChatCommand({"/kotj"}, options)
	
	self:InitDB()
	
	self:Print("Rawrrrrrr...")
end

function KingOfTheJungle:OnEnable()	
	self.feedButton = CreateFrame("Button", "KOTJFeedButton", UIParent, "SecureActionButtonTemplate,ActionButtonTemplate")					
	self.feedButton:SetPoint(self.db.char.buttonPoint,self.db.char.buttonX,self.db.char.buttonY) 	
	KOTJFeedButtonIcon:SetTexture("Interface\\Icons\\Ability_Hunter_BeastTraining")
	self.feedButton:Hide()
	self.feedButton:SetMovable(1)
	self.feedButton:RegisterForDrag("LeftButton")
	self.feedButton:SetScale(self.db.char.buttonScale)
	
	self.feedButton:SetAttribute("unit", "player")
	self.feedButton:SetAttribute("type", "spell")
	self.feedButton:SetAttribute("spell", BS["Feed Pet"])
	self.feedButton:SetAttribute("target-item", ATTRIBUTE_NOOP)
			
	self.feedButton:SetScript("OnEnter", function()
		local itemlink = self:FeedPet()
		if ( itemlink ) then
			GameTooltip:SetOwner(this,"ANCHOR_RIGHT")
			GameTooltip:SetHyperlink(itemlink)
			GameTooltip:Show()
		else
			GameTooltip:SetOwner(this,"ANCHOR_RIGHT")
			GameTooltip:SetText(L["No food available for your pet."], 1.0,1.0,1.0)
			GameTooltip:Show()
		end			
	end)
	
	self.feedButton:SetScript("OnLeave", function()
		GameTooltip:Hide()		
	end)
	
--~ 	Make a more smoother fix for this
	self.feedButton:SetScript("PreClick", function()
		local itemlink = self:FeedPet()
		if not itemlink then									
			if UnitName("pet") then
				KingOfTheJungle:Chat(string.format(L["%s looks in your bags and sees no food in there."],UnitName("pet")))
			end
		end
	end)
	
	self.feedButton:SetScript("PostClick", function()
		self:HideFeedPetButton()	
	end)	
	
	self.feedButton:SetScript("OnDragStart", function()
		this:StartMoving()
	end)
	
	self.feedButton:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		ValidateFramePosition(this)
		self.db.char.buttonPoint,_,_,self.db.char.buttonX,self.db.char.buttonY = this:GetPoint()
	end);
		
	_, englishClass = UnitClass("player")
	
	if englishClass == "HUNTER" then				
		self:RegisterEvent("UNIT_HAPPINESS") 
		self:RegisterEvent("CHAT_MSG_SPELL_FAILED_LOCALPLAYER")	

		self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
		self:RegisterEvent("PLAYER_CONTROL_LOST")
		self:RegisterEvent("PLAYER_REGEN_DISABLED")			
		
		local event = "CHAT_MSG_SPELL_TRADESKILLS"
		PL:RegisterEvent("KOTJ", event, function(event, info) 							
							if info.type == 'feedpet' and info.victim == ParserLib_SELF then	
								KingOfTheJungle:Chat(string.format(L["%s loves %s, mjummie."],UnitName("pet"),info.item))
							end
						end)
	end
end

function KingOfTheJungle:OnDisable()    
end

function KingOfTheJungle:UNIT_SPELLCAST_SUCCEEDED()
	if arg1 == "player" and arg2 == BS["Dismiss Pet"] then
		self:HideFeedPetButton()
	end	
end

function KingOfTheJungle:PLAYER_CONTROL_LOST()
	self:HideFeedPetButton()
end

function KingOfTheJungle:PLAYER_REGEN_DISABLED()
	if self.feedButton:IsShown() then
		self:HideFeedPetButton()
	end
end

function KingOfTheJungle:CHAT_MSG_SPELL_FAILED_LOCALPLAYER()
	if (arg1 and string.find(arg1, L["You fail to perform Feed Pet: You are mounted."])) then
		KingOfTheJungle:Chat(string.format(L["%s thinks you should unmount before feeding him."],UnitName("pet")))
	end
	if (arg1 and string.find(arg1, L["You fail to perform Feed Pet: Out of range."])) then
		KingOfTheJungle:Chat(string.format(L["%s likes to get closer with you before getting fed."],UnitName("pet")))
	end
	if (arg1 and string.find(arg1, L["You fail to perform Feed Pet: That food's level is not high enough for your pet."])) then
		KingOfTheJungle:Chat(string.format(L["%s wants food of a more appropriate level."],UnitName("pet")))
	end
	if (arg1 and string.find(arg1, L["You fail to perform Feed Pet: Can't do that while stunned."])) then
		KingOfTheJungle:Chat(string.format(L["%s looks at you, wondering how you will feed him when you are stunned."],UnitName("pet")))
	end
end

function KingOfTheJungle:UNIT_HAPPINESS()	
	if not self.db.char.warnPetHungry then
		return
	end	

	if InCombatLockdown() then
		return 
	end
	
	if UnitHealth("pet") <= 0 or UnitIsDeadOrGhost("pet") then
		self:HideFeedPetButton()
		return
	end
	
	
--~     Number - the numerical happiness value of the pet (1 = unhappy, 2 = content, 3 = happy)
	local happiness, _, _ = GetPetHappiness()	
	
	if happiness and happiness <= self.db.char.petHappinessLevel then
		if self.db.char.warnPetHungry then
			if not self:CheckForBuffByPath(L["Feed Pet Effect"]) then
				if self:FeedPet()~=nil then
					self.feedButton:Show()
					return
				else					
					if difftime(time(), self.db.char.lastWarningTime) > self.db.char.msgInterval then
						self.db.char.lastWarningTime = time()
						KingOfTheJungle:Chat(string.format(L["%s looks in your bags and sees no food in there."],UnitName("pet")))
					end
				end
			end
		end
	end	
	
	self:HideFeedPetButton()
end

function KingOfTheJungle:Chat(text)
	DEFAULT_CHAT_FRAME:AddMessage(string.format("|cffeda55f %s |r", tostring(text)))	
end

function KingOfTheJungle:CheckForBuffByPath(texture)
	local i = 1;	
	local _, _, buffTexture, _ = UnitBuff("pet",i)	
	while buffTexture do
		if string.find(texture, buffTexture) then
			return true
		end
		i = i + 1
		_, _, buffTexture, _ = UnitBuff("pet",i)
	end
	return false
end

function KingOfTheJungle:FeedPet()	
	local diets = {GetPetFoodTypes()}
	
	for i,_ in ipairs(self.db.profile.foodlist) do
		for idx, _ in ipairs(diets) do
			if string.find(self.db.profile.foodlist[i]["diet"],L:GetReverseTranslation(diets[idx])) then 
				for index,_ in pairs(self.db.profile.foodlist[i]["list"]) do
					itemName, itemLink, _, itemLevel, _, _, _, _, _, _ = GetItemInfo(self.db.profile.foodlist[i]["list"][index]["id"])
					if itemLink and GetItemCount(itemLink) > 0 and self.db.profile.foodlist[i]["list"][index]["selected"] then			
						self.feedButton:SetAttribute("target-item", itemName)
						return itemLink
					end
				end
			end
		end
	end	
	self.feedButton:SetAttribute("target-item", ATTRIBUTE_NOOP)
	return nil
end

function KingOfTheJungle:HideFeedPetButton()
	if not (GetPetHappiness() == 3 and self.db.char.petHappinessLevel == 3) then
		self.feedButton:Hide()
	end
end